######################################################################
# Creation 02/2020 Francesc Roura Adserias (froura), adapted from "S2S4E-backend-BSC/data-analysis/create_mask.R"
# ====================================================================
# This script converts a shapefile (with one or multiple polygons) 
# to a mask to be used in other scripts that use global lat lon files.
######################################################################

library(ncdf4)
library(easyNCDF)
library(rgdal)

lat <- NcToArray('/esarchive/oper/S2S4E-data/weekly_statistics/tas/tas_20170606.nc', vars_to_read='latitude')
lon <- NcToArray('/esarchive/oper/S2S4E-data/weekly_statistics/tas/tas_20170606.nc', vars_to_read='longitude')
lat <- lat[1,]
lon <- lon[1,]
lon <- ((lon + 180) %% 360) - 180

shp <- rgdal::readOGR("/home/Earth/froura/repositories/auto-s2s/data-analysis/shapefiles/regions-polygon.shp")

npol <- length(attributes(shp)$polygons) #number of polygons into the shapefile

region.mask <- array(dim = c(npol, 190, 384))
region.names <- c("reg1", "reg2")
count <- 0

for(reg in 1:npol){
  for (latitude in 1:length(lat)){
    for (longitude in 1:length(lon)){
    
      region.mask[reg,latitude,longitude] <- point.in.polygon(lat[latitude],
                                                           lon[longitude],
                                                           slot(slot(attributes(shp)$polygons[[reg]],"Polygons")[[1]],"coords")[,1], #lat
                                                           slot(slot(attributes(shp)$polygons[[reg]],"Polygons")[[1]],"coords")[,2]  #lon
      )
      if(region.mask[reg,latitude,longitude]==2){
        region.mask[reg,latitude,longitude]==1
        warning("Point close to the border of the polygon. Value 2 converted to 1")
      }
      if(region.mask[reg,latitude,longitude]==3){
        region.mask[reg,latitude,longitude]==1
        warning("Point at the vertex of the polygon. Value 3 converted to 1")
      }
    }
    count <- count +1 
    # print(count)
  }
  print(paste(substr(as.character(reg/npol*100),0,3),"%"))
}
print("####### MASK DONE ######")

save(region.mask, region.names, file="/home/Earth/froura/repositories/auto-s2s/data-analysis/masks/mask_region_decathlon.Rdata")

